@@ -17,13 +17,14 @@
 
 import urllib.request as urllib2
 import socket
-import re
 import ssl
+import logging
+import os
 
 from gajim.common import app
-import logging
+from gajim.plugins.plugins_i18n import _
+
 
-import os
 if os.name == 'nt':
     import certifi
 
