@@ -21,6 +21,7 @@ import binascii
 import logging
 import math
 from urllib.parse import urlparse
+from urllib.parse import unquote
 from io import BytesIO
 import shutil
 from functools import partial
@@ -40,19 +41,20 @@ from gajim.plugins.plugins_i18n import _
 
 from url_image_preview.http_functions import get_http_head, get_http_file
 from url_image_preview.config_dialog import UrlImagePreviewConfigDialog
-from url_image_preview.resize_gif import resize_gif
 
 from gajim.gtk.filechoosers import FileSaveDialog
+from gajim.gtk.util import get_cursor
 
 
 log = logging.getLogger('gajim.plugin_system.preview')
 
-PILLOW_AVAILABLE = True
+ERROR_MSG = None
 try:
     from PIL import Image
+    from url_image_preview.resize_gif import resize_gif
 except:
     log.debug('Pillow not available')
-    PILLOW_AVAILABLE = False
+    ERROR_MSG = 'Please install python-pillow'
 
 try:
     if os.name == 'nt':
@@ -77,6 +79,12 @@ ACCEPTED_MIME_TYPES = ('image/png', 'image/jpeg', 'image/gif', 'image/raw',
 class UrlImagePreviewPlugin(GajimPlugin):
     @log_calls('UrlImagePreviewPlugin')
     def init(self):
+        if ERROR_MSG:
+            self.activatable = False
+            self.available_text = ERROR_MSG
+            self.config_dialog = None
+            return
+
         if not decryption_available:
             self.available_text = DEP_MSG
         self.config_dialog = partial(UrlImagePreviewConfigDialog, self)
@@ -174,7 +182,7 @@ class Base(object):
             log.debug('Url with text will not be displayed: %s', real_text)
             return
 
-        urlparts = urlparse(real_text)
+        urlparts = urlparse(unquote(real_text))
         if not self._accept_uri(urlparts, real_text, additional_data):
             return
 
@@ -245,6 +253,12 @@ class Base(object):
             filename = os.path.basename(urlparts.path)
             ext = os.path.splitext(filename)[1]
             name = os.path.splitext(filename)[0]
+            if len(name) > 90:
+                # Many Filesystems have a limit on filename length
+                # Most have 255, some encrypted ones only 143
+                # We add around 50 chars for the hash,
+                # so the filename should not exceed 90
+                name = name[:90]
             namehash = hashlib.sha1(real_text.encode('utf-8')).hexdigest()
             newfilename = name + '_' + namehash + ext
             thumbfilename = name + '_' + namehash + '_thumb_' \
@@ -353,9 +367,6 @@ class Base(object):
             log.info('Failed to load image using Gdk.Pixbuf')
             log.debug(error)
 
-            if not PILLOW_AVAILABLE:
-                log.info('Pillow not available')
-                return
             # Try Pillow
             image = Image.open(BytesIO(mem)).convert("RGBA")
             array = GLib.Bytes.new(image.tobytes())
@@ -379,12 +390,12 @@ class Base(object):
                     width, height, GdkPixbuf.InterpType.BILINEAR)
                 thumbnail.savev(thumbpath, 'png', [], [])
         except Exception as error:
-            dialogs.ErrorDialog(
+            GLib.idle_add(
+                self._raise_error_dialog,
                 _('Could not save file'),
                 _('Exception raised while saving thumbnail '
                   'for image file (see error log for more '
-                  'information)'),
-                transient_for=app.app.get_active_window())
+                  'information)'))
             log.exception(error)
             return
         return thumbnail
@@ -447,17 +458,11 @@ class Base(object):
             log.error('Could not download image for URL: %s', url)
             return
 
-        urlparts = urlparse(url)
+        urlparts = urlparse(unquote(url))
         filename = os.path.basename(urlparts.path)
         if os.path.basename(filepath).startswith('location_'):
             filename = os.path.basename(filepath)
 
-        event_box = Gtk.EventBox()
-        event_box.connect('button-press-event', self.on_button_press_event,
-                          filepath, filename, url, encrypted)
-        event_box.connect('enter-notify-event', self.on_enter_event)
-        event_box.connect('leave-notify-event', self.on_leave_event)
-
         def add_to_textview():
             try:
                 at_end = self._get_at_end()
@@ -468,24 +473,17 @@ class Base(object):
                 anchor = buffer_.create_child_anchor(iter_)
                 anchor.plaintext = url
 
-                if isinstance(pixbuf, GdkPixbuf.PixbufAnimation):
-                    image = Gtk.Image.new_from_animation(pixbuf)
-                else:
-                    image = Gtk.Image.new_from_pixbuf(pixbuf)
+                image = self._create_clickable_image(pixbuf, url)
 
-                css = '''#Preview {
-                box-shadow: 0px 0px 3px 0px alpha(@theme_text_color, 0.2);
-                margin: 5px 10px 5px 10px; }'''
-                gtkgui_helpers.add_css_to_widget(image, css)
-                image.set_name('Preview')
-
-                event_box.set_tooltip_text(url)
-                event_box.add(image)
-                event_box.show_all()
-                self.textview.tv.add_child_at_anchor(event_box, anchor)
+                self.textview.tv.add_child_at_anchor(image, anchor)
                 buffer_.delete(iter_,
                                buffer_.get_iter_at_mark(repl_end))
 
+                image.connect(
+                    'button-press-event', self.on_button_press_event,
+                    filepath, filename, url, encrypted)
+                image.get_window().set_cursor(get_cursor('HAND2'))
+
                 if at_end:
                     self._scroll_to_end()
             except Exception as ex:
@@ -494,6 +492,24 @@ class Base(object):
         # add to mainloop --> make call threadsafe
         GLib.idle_add(add_to_textview)
 
+    def _create_clickable_image(self, pixbuf, url):
+        if isinstance(pixbuf, GdkPixbuf.PixbufAnimation):
+            image = Gtk.Image.new_from_animation(pixbuf)
+        else:
+            image = Gtk.Image.new_from_pixbuf(pixbuf)
+
+        css = '''#Preview {
+        box-shadow: 0px 0px 3px 0px alpha(@theme_text_color, 0.2);
+        margin: 5px 10px 5px 10px; }'''
+        gtkgui_helpers.add_css_to_widget(image, css)
+        image.set_name('Preview')
+
+        event_box = Gtk.EventBox()
+        event_box.set_tooltip_text(url)
+        event_box.add(image)
+        event_box.show_all()
+        return event_box
+
     def _check_mime_size(self, tuple_arg,
                          url, weburl, repl_start, repl_end, filepaths,
                          key, iv, encrypted):
@@ -545,11 +561,11 @@ class Base(object):
             # Write file to harddisk
             self._write_file(filepath, mem)
         except Exception as e:
-            dialogs.ErrorDialog(
+            GLib.idle_add(
+                self._raise_error_dialog,
                 _('Could not save file'),
                 _('Exception raised while saving image file'
-                  ' (see error log for more information)'),
-                transient_for=app.app.get_active_window())
+                  ' (see error log for more information)'))
             log.error(str(e))
 
         # Create thumbnail, write it to harddisk and return it
@@ -683,17 +699,6 @@ class Base(object):
         except Exception:
             pass
 
-    # Change mouse pointer to HAND2 when
-    # mouse enter the eventbox with the image
-    def on_enter_event(self, eb, event):
-        self.textview.tv.get_window(
-            Gtk.TextWindowType.TEXT).set_cursor(Gdk.Cursor(Gdk.CursorType.HAND2))
-
-    # Change mouse pointer to default when mouse leaves the eventbox
-    def on_leave_event(self, eb, event):
-        self.textview.tv.get_window(
-            Gtk.TextWindowType.TEXT).set_cursor(Gdk.Cursor(Gdk.CursorType.XTERM))
-
     def on_button_press_event(self, eb, event, filepath,
                               original_filename, url, encrypted):
         data = {"filepath": filepath,
@@ -713,5 +718,12 @@ class Base(object):
             # menu.popup(None, None, None, event.button, event.time)
             menu.popup_at_pointer(event)
 
+    @staticmethod
+    def _raise_error_dialog(pritext, sectext):
+        # Used by methods that run in a different thread
+        dialogs.ErrorDialog(pritext,
+                            sectext,
+                            transient_for=app.app.get_active_window())
+
     def disconnect_from_chat_control(self):
         pass
